package net.cattaka.rdbassistant.mysql;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import net.cattaka.rdbassistant.core.SqlEditorSelection;
import net.cattaka.rdbassistant.gui.table.ResultSetTableModel;
import net.cattaka.rdbassistant.gui.table.StaticResultSetTableModel;
import net.cattaka.util.ExceptionHandler;

public class MySqlSqlEditorSelection implements SqlEditorSelection {
	public static String SELECT_DATABASE_LIST = "SELECT DISTINCT TABLE_SCHEMA FROM INFORMATION_SCHEMA.TABLES ORDER BY TABLE_SCHEMA";
	public static String SELECT_TABLES_LIST = "SELECT TABLE_NAME, TABLE_COMMENT, TABLE_CATALOG, TABLE_TYPE, ENGINE, VERSION, ROW_FORMAT, TABLE_ROWS, AVG_ROW_LENGTH, DATA_LENGTH, MAX_DATA_LENGTH, INDEX_LENGTH, DATA_FREE, AUTO_INCREMENT, CREATE_TIME, UPDATE_TIME, CHECK_TIME, TABLE_COLLATION, CHECKSUM, CREATE_OPTIONS FROM information_schema.TABLES WHERE TABLE_SCHEMA=? ORDER BY TABLE_NAME";
	public static String SELECT_VIEWS_LIST = "SELECT TABLE_NAME, TABLE_CATALOG, VIEW_DEFINITION, CHECK_OPTION, IS_UPDATABLE, DEFINER, SECURITY_TYPE FROM information_schema.VIEWS WHERE TABLE_SCHEMA=? ORDER BY TABLE_NAME";
	public static String[] SELECT_DETAIL_NAME_LIST = {
		"COLUMNS",
		"CONSTRAINTS"
	};
	public static String[] SELECT_DETAIL_SQL_LIST = {
		"SELECT COLUMN_NAME, COLUMN_TYPE, COLUMN_COMMENT, IS_NULLABLE, TABLE_CATALOG, ORDINAL_POSITION, COLUMN_DEFAULT, DATA_TYPE, CHARACTER_MAXIMUM_LENGTH, CHARACTER_OCTET_LENGTH, NUMERIC_PRECISION, NUMERIC_SCALE, CHARACTER_SET_NAME, COLLATION_NAME, COLUMN_KEY, EXTRA, PRIVILEGES FROM information_schema.COLUMNS WHERE TABLE_SCHEMA=? AND TABLE_NAME=? ORDER BY ORDINAL_POSITION",
		"SELECT CONSTRAINT_NAME, CONSTRAINT_TYPE, CONSTRAINT_CATALOG, CONSTRAINT_SCHEMA FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_SCHEMA=? AND TABLE_NAME=? ORDER BY CONSTRAINT_NAME"
	};
	
	private MySqlRdbaConnection rdbConnection;
	
	public MySqlSqlEditorSelection(MySqlRdbaConnection rdbConnection) {
		this.rdbConnection = rdbConnection;
	}

	public String getDefaultDatabase() {
		return rdbConnection.getDefaultDatabase();
	}

	public List<String> getDatabaseList() {
		String sqlStr = SELECT_DATABASE_LIST;
		ArrayList<String> result = new ArrayList<String>();
		try {
			PreparedStatement stmt = rdbConnection.getConnection().prepareStatement(sqlStr);
			ResultSet rs = stmt.executeQuery();
			while(rs.next()) {
				result.add(rs.getString(1));
			}
			stmt.close();
		} catch (SQLException e) {
			ExceptionHandler.error(e);
		}
		return result;
	}

	public List<String> getObjectTypeList() {
		ArrayList<String> result = new ArrayList<String>();
		result.add("TABLE");
		result.add("VIEW");
		return result;
	}
	
	public ResultSetTableModel getTableList(String database, String objectType) {
		String sqlStr = null;
		if (objectType.equals("TABLE")) {
			sqlStr = SELECT_TABLES_LIST;
		} else if (objectType.equals("VIEW")) {
			sqlStr = SELECT_VIEWS_LIST;
		}
		StaticResultSetTableModel result = new StaticResultSetTableModel();
		if (sqlStr != null) {
			PreparedStatement stmt = null;
			ResultSet rs = null;
			try {
				stmt = rdbConnection.getConnection().prepareStatement(sqlStr);
				stmt.setString(1, database);
				rs = stmt.executeQuery();
				result.extractResultSetData(rdbConnection, rs);
			} catch (SQLException e) {
				ExceptionHandler.error(e);
			} finally {
				try {
					if (rs != null) {
						rs.close();
					}
				} catch (SQLException e) {
					ExceptionHandler.warn(e);
				}
				try {
					if (stmt != null) {
						stmt.close();
					}
				} catch (SQLException e) {
					ExceptionHandler.warn(e);
				}
			}
		}
		return result;
	}

	public List<ResultSetTableModel> getTableProperties(String database,String objectType, String table) {
		ArrayList<ResultSetTableModel> result = new ArrayList<ResultSetTableModel>();
		for (int i=0;i<SELECT_DETAIL_NAME_LIST.length;i++) {
			PreparedStatement stmt = null;
			ResultSet rs = null;
			String sqlStr = String.format(SELECT_DETAIL_SQL_LIST[i], database, table);
			try {
				StaticResultSetTableModel pt = new StaticResultSetTableModel();
				// 実テーブルの列情報
				stmt = rdbConnection.getConnection().prepareStatement(sqlStr);
				stmt.setString(1, database);
				stmt.setString(2, table);
				rs = stmt.executeQuery();
				
				pt.extractResultSetData(rdbConnection, rs);
				pt.setTableName(SELECT_DETAIL_NAME_LIST[i]);
				
				result.add(pt);
			} catch (SQLException e) {
				ExceptionHandler.error(e);
			} finally {
				try {
					if (rs != null) {
						rs.close();
					}
				} catch (SQLException e) {
					ExceptionHandler.warn(e);
				}
				try {
					if (stmt != null) {
						stmt.close();
					}
				} catch (SQLException e) {
					ExceptionHandler.warn(e);
				}
			}
		}
		return result;
	}
		
	public String[] getColumnDefNameAsArray() {
		// TODO : ローカライズ
		return new String[]{"COLUMN_NAME", "TYPE", "NULLABLE", "PRIMARY KEY", "DEFUALT_VALUE", "COMMENT"};
	}
}
